﻿#include "strdef.h"
#include "rc.h"

//
//トラックバーの作成
//hWnd...メインウィンドウのハンドル
//
//return 成功時TRUE / 失敗時FALSE
//
BOOL CreateTrackbar(HWND hWnd,InternalData *temp)
{
	long  lFirstPosition = 0;
	long  lPageSize = 5;
	HWND hTrackbar,hSoundControll;

	/* コモンコントロールの初期化 */
	InitCommonControls();

	/* トラックバーコントロールの作成 */
	hTrackbar = CreateWindowExW(
		0, TRACKBAR_CLASS, NULL,
		WS_CHILD | WS_VISIBLE |
		TBS_NOTICKS | TBS_TOOLTIPS,
		0, 0, 200, 30,
		hWnd, (HMENU)DF_IDC_TRACKBAR_PLAY, (HINSTANCE)GetWindowLongPtr(hWnd, GWLP_HINSTANCE), NULL
	);

	hSoundControll = CreateWindowExW(
		0, TRACKBAR_CLASS, NULL,
		WS_CHILD | WS_VISIBLE | TBS_VERT |
		TBS_NOTICKS | TBS_TOOLTIPS,
		0, UI_HEIGHT * 3 - 10, BUTTON_WIDE + 5, 0,/* HEIGHT - UI_HEIGHT * 3*/
		hWnd, (HMENU)DF_IDC_TRACKBAR_SOUND, (HINSTANCE)GetWindowLongPtr(hWnd, GWLP_HINSTANCE), NULL
	);

	if (!hTrackbar || !hSoundControll)
#if _DEBUG
		return FALSE;
#else
		exit(0);
#endif

	/* 範囲のセット */
	SendMessageW(hTrackbar, TBM_SETRANGE, (WPARAM)TRUE, (LPARAM)MAKELONG(0, 0));

	/* 1ページのサイズのセット */
	SendMessageW(hTrackbar, TBM_SETPAGESIZE, 0, (LPARAM)lPageSize);

	/* 初期値のセット */
	SendMessageW(hTrackbar, TBM_SETPOS, (WPARAM)TRUE, (LPARAM)lFirstPosition);

	//サウンドに関するウィンドウは見えないように
	ShowWindow(hSoundControll, SW_HIDE);

	//トラックバーの背景の透過

	OutputExeLogA("処理完了。\n");

	temp->hTrack_Sound.hTrackbar = hSoundControll;
	temp->hTrack_Play.hTrackbar = hTrackbar;

	Thread_StartCollision(temp);

	return TRUE;
}

/*
//
//トラックバーに関する設定を行う
//indata...設定されるべきトラックバーのハンドル
//
void SetTrackbarStatus(InternalData indata)
{
	// 範囲のセット 
	SendMessageW(indata.hTrack_Play.hTrackbar, TBM_SETRANGE, (WPARAM)TRUE, (LPARAM)MAKELONG(0, indata.maxSrc));
}
*/

//
//Userに開かれるべきファイルを訪ねる
//hWnd...親ウィンドのハンドル
//temp...情報を受け取ることができる構造体
//
//return TRUE...成功 /FALSE...失敗
//
int GetFileNames(HWND hWnd, InternalData *temp)
{
	const long MAXFILELENGTH = MAX_PATH * 34;

	OPENFILENAMEW ofn;
	WCHAR fileName[MAXFILELENGTH] = { 0 };
	WCHAR strCustom[256] = L"Before files\0*.*\0\0";
	size_t iCnt = 0;
	size_t nPos;
	size_t Dirlen;

	ZeroMemory(&ofn, OPENFILENAME_SIZE_VERSION_400);

	SuspendThread(temp->th.hThread);

	ofn.lStructSize = OPENFILENAME_SIZE_VERSION_400;
	ofn.hwndOwner = hWnd;
	ofn.lpstrFilter = L"All Files(*.*)\0*.*\0\0";
	ofn.lpstrCustomFilter = strCustom;
	ofn.nMaxCustFilter = 256;
	ofn.nFilterIndex = 0;
	ofn.lpstrFile = fileName;
	ofn.nMaxFile = MAXFILELENGTH;
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT | OFN_EXPLORER;
	ofn.lpstrTitle = L"開く";

	OutputExeLogA("初期化完了\n");

	IF (GetOpenFileNameW(&ofn))
	{
		nPos = wcslen(fileName) + 1;
		iCnt = 0;

		//とりあえずディレクトリを割り出す
		Dirlen = wcslen(fileName) + 1;
		IF ((temp->ParentDir = (WCHAR*)malloc(Dirlen * sizeof(WCHAR))) == NULL)
			return FALSE;
		//残りメモリも確保しておく(多くても損はしない)
		IF ((temp->File = (WCHAR**)malloc(MAXFILELENGTH * sizeof(WCHAR))) == NULL)
		{
			free(temp->ParentDir);
			return FALSE;
		}

		ZeroMemory(temp->File, MAXFILELENGTH * sizeof(WCHAR));

		//データをコピー
		//(strcpy_sだとメモリをよこせと言ってくるので)
		memcpy(temp->ParentDir, fileName, Dirlen * sizeof(WCHAR));

//		OutputExeLog(L"選択されたファイル名を列挙します。\n");

		IF (fileName[nPos])
		{
//			OutputExeLog(L"ファイルのディレクトリ:%s\\\n", temp->ParentDir);

			// 複数ファイルが選択された
			do {
				// パスをコピー
				if (fileName[nPos]) temp->File[iCnt] = (WCHAR*)malloc((wcslen(&fileName[nPos]) + 1) * sizeof(WCHAR));
				else break;
				if (!temp->File[iCnt])
				{
					//leak
					free(temp->File);
					free(temp->ParentDir);

					return FALSE;
				}
				wcscpy_s(temp->File[iCnt], (wcslen(&fileName[nPos]) + 1), &fileName[nPos]);
//				OutputExeLog(L"ファイル名:%s\n", temp->File[iCnt]);
				iCnt++;
				nPos += (wcslen(&fileName[nPos]) + 1);
			} while (fileName[nPos]);
		}
		else {
			//親ディレクトリおよびファイルに関するメモリを確保しなおす
			ZeroMemory(temp->ParentDir + (Dirlen - wcslen(PathFindFileNameW(fileName)) - 2/*NULL + \\*/), sizeof(WCHAR));
			//親ディレクトリ名だけを確保
			temp->File[0] = (WCHAR*)malloc(MAXFILELENGTH * sizeof(WCHAR));
			if (!temp->File[0])
				return FALSE;
			wcscpy_s(temp->File[0], wcslen(PathFindFileNameW(fileName)) + 1, PathFindFileNameW(fileName));
			iCnt = 1;

//			OutputExeLog(L"ファイルのディレクトリ:%s\\\n", temp->ParentDir);
//			OutputExeLog(L"ファイル名:%s\n", temp->File[0]);
		}
	}
	else
	{
		ResumeThread(temp->th.hThread);
		return FALSE;
	}
	//準備
	ResumeThread(temp->th.hThread);
	temp->NowPlay = 0;
	temp->MaxPlay = (int)iCnt;
	return TRUE;
}

//
//ボタンを作成
//hWnd...メインウィンドウのハンドル
//
//return 作成されたボタンのハンドル / 失敗時：NULL
//
int CreateButton(HWND hWnd,InternalData *temp)
{
	HINSTANCE hInstance;

	hInstance = (HINSTANCE)GetWindowLongPtrW(hWnd, GWLP_HINSTANCE);

	//再生/停止
	temp->hButton_Play.hButton = CreateWindowExW(
		0, WC_BUTTONW, TEXT_BUTTON_PLAY,
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		0, 0, 30, 25,
		hWnd, (HMENU)BT_IDC_MCIBUTTON_PLAY, hInstance, NULL
	);

	//音量
	temp->hButton_Sound.hButton = CreateWindowExW(
		0, WC_BUTTONW, NULL,
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON | BS_OWNERDRAW,
		0, 0, 30, 25,
		hWnd, (HMENU)BT_IDC_MCIBUTTON_SOUND, hInstance, NULL
	);

	if (!temp->hButton_Play.hButton || !temp->hButton_Sound.hButton)
		return FALSE;

	//ウィンドウプロシージャの乗っ取り
	temp->hButton_Sound.DefaultProc = (WNDPROC)GetWindowLongPtrW(temp->hButton_Sound.hButton, GWLP_WNDPROC);
	SetWindowLongPtrW(temp->hButton_Sound.hButton, GWLP_WNDPROC, (LONG_PTR)(WNDPROC)CustomDefButtonProc);
	//先にコピー
	temp->hButton_Sound.hIcon = LoadIconW(hInstance, L"SOUND");

	RegisterButtonInfo(temp->hButton_Sound, NULL, NULL, FALSE);

	OutputExeLogA("処理完了。\n");

	return TRUE;
}

//
//ボタンコントロールの値を保持する
//
//button ...　保存したいボタンの情報
//hWnd...得たい情報のhwnd(登録するときは必ずNULL)
//*bc...得たい情報
//DeleteFlag ... 削除するかどうか
//
//return ... 成功時/TRUE 失敗時/FALSE
//
int RegisterButtonInfo(BC button,HWND hWnd,BC *bc,int DeleteFlag)
{
	static size_t count, i;
	static BC *_bc, *temp;

	//
	//削除
	//
	if (DeleteFlag)
	{
		free(_bc);
		return TRUE;
	}

	//
	//取得
	//
	if (hWnd)
	{
		for (i = 0; i < count; i++)
		{
			if (_bc[i].hButton == hWnd)
			{
				*bc = _bc[i];
				return TRUE;
			}

			bc->DefaultProc = NULL;
			bc->hButton = NULL;
			bc->NewProc = NULL;

			return FALSE;
		}
	}

	//
	//メモリの確保
	//

	OutputExeLogA("\'%s\' は \'%d\'として評価されました。\n", OPTP(count), count);

	if (!count)
		_bc = (BC*)malloc(sizeof(BC));
	else
	{
		temp = (BC*)realloc(bc, sizeof(BC)*(count + 1));
		if (!temp)
			return FALSE;
	}

	if (!_bc)
		return FALSE;
	if (temp)
		_bc = temp;

	//
	//登録
	//
	_bc[count] = button;
	count++;

	OutputExeLogA("処理完了。\n");

	//終了
	return TRUE;

}

//
//バージョン情報の表示
//
VOID ShowVersionInfo(HWND hwnd)
{
	ShellAboutW(
		hwnd,
		L"バージョン情報#Version:" APPLICATION_VERSION L"(" ORIGINAL_NAME L")",
		COPY_RIGHT,
		(HICON)GetClassLongPtrW(hwnd, GCLP_HICON)
	);
}
